local moduleName = 'Questing'
local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local module = idTweaks:newModule(moduleName, 'AceConsole-3.0', 'AceEvent-3.0')

module.defaults = {
  profile = {
    enabled = true,

    pauseKey = 'shift',
    acceptQuests = true,
    completeQuests = true,
  }
}

module.options = {
  name = 'Questing',
  desc = 'Options for questing.',
  type = 'group',
  args = {
    enabled = {
      order = 1,
      name = 'Enabled',
      desc = 'Enables / Disables the Module',
      type = 'toggle',
      get = function()
        return module.db.profile.enabled
      end,
      set = function(_, value)
        module.db.profile.enabled = value
        module:check()
      end,
    },

    pauseKey = {
      name = 'Pause Key',
      desc = 'While you hold this key, nothing will happen, so you can read your quest in peace.',
      type = 'select',
      values = {
        shift = 'Shift',
        control = 'Control',
        alt = 'Alt',
      },
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.pauseKey
      end,
      set = function(_, value)
        module.db.profile.pauseKey = value
      end
    },

    acceptQuests = {
      name = 'Accept Quests',
      desc = 'Automatically accept all quests. Might need a few clicks to get them all.',
      type = 'toggle',
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.acceptQuests
      end,
      set = function(_, value)
        module.db.profile.acceptQuests = value
      end
    },

    completeQuests = {
      name = 'Complete quests',
      desc = 'Automatically complete all quests that dont require you to pick a reward.',
      type = 'toggle',
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.completeQuests
      end,
      set = function(_, value)
        module.db.profile.completeQuests = value
      end
    }
  },
}

function module:enable()
  module:RegisterEvent('GOSSIP_SHOW')
  module:RegisterEvent('QUEST_AUTOCOMPLETE')
  module:RegisterEvent('QUEST_COMPLETE')
  module:RegisterEvent('QUEST_DETAIL')
  module:RegisterEvent('QUEST_GREETING')
  module:RegisterEvent('QUEST_LOG_UPDATE')
  module:RegisterEvent('QUEST_PROGRESS')
end

function module:disable()
  module:UnregisterEvent('GOSSIP_SHOW')
  module:UnregisterEvent('QUEST_AUTOCOMPLETE')
  module:UnregisterEvent('QUEST_COMPLETE')
  module:UnregisterEvent('QUEST_DETAIL')
  module:UnregisterEvent('QUEST_GREETING')
  module:UnregisterEvent('QUEST_LOG_UPDATE')
  module:UnregisterEvent('QUEST_PROGRESS')
end

function module:shouldAutomate()
  local output = true
  local key = module.db.profile.pauseKey

  if key == 'shift' then
    if IsShiftKeyDown() then
      output = false
    end
  end

  if key == 'control' then
    if IsControlKeyDown() then
      output = false
    end
  end

  if key == 'alt' then
    if IsAltKeyDown() then
      output = false
    end
  end

  return output
end

function module:shouldAccept()
  local output = not not module.db.profile.acceptQuests

  return output
end

function module:shouldComplete()
  local output = not not module.db.profile.completeQuests

  return output
end

function module:stripText(text)
  if not text then return end
  text = text:gsub('|c%x%x%x%x%x%x%x%x(.-)|r','%1')
  text = text:gsub('%[.*%]%s*','')
  text = text:gsub('(.+) %(.+%)', '%1')
  text = text:trim()
  return text
end

function module:QUEST_AUTOCOMPLETE(event, questId)
  if not module:shouldAutomate() then return end
  if not module:shouldComplete() then return end
  for i = 0, 6 do
    local id = GetQuestIndexForWatch(i)
    if id then
      ShowQuestComplete(id)
    end
  end
end

function module:QUEST_COMPLETE(event)
  if not module:shouldAutomate() then return end
  if not module:shouldComplete() then return end
  if GetNumQuestChoices() <= 1 then
    QuestFrameCompleteQuestButton:Click()
  end
end

function module:QUEST_PROGRESS()
  if not module:shouldAutomate() then return end
  if not module:shouldComplete() then return end
  if IsQuestCompletable() then
    CompleteQuest()
  end
end

function module:QUEST_LOG_UPDATE()
  local startEntry = GetQuestLogSelection()
  local numEntries = GetNumQuestLogEntries()
  local title
  local isComplete
  local noObjectives
  local _

  module.completedQuests = {}
  module.incompleteQuests = {}

  if numEntries > 1 then
    for i = 1, numEntries do
      SelectQuestLogEntry(i)
      title, _, _, _, _, isComplete = GetQuestLogTitle(i)

      if title then
        if isComplete then
          module.completedQuests[title] = true
        else
          module.incompleteQuests[title] = true
        end
      end
    end
  end

  SelectQuestLogEntry(startEntry)
end

function module:GOSSIP_SHOW()
  if not module:shouldAutomate() then return end

  local button
  local text

  if
    GossipTitleButton1:IsVisible() and
    (
      GossipTitleButton1.type == 'Gossip' or
      GossipTitleButton1.type == 'Active'
    ) and
    not GossipTitleButton2:IsVisible()
  then
    GossipTitleButton1:Click()
    return
  end

  for i = 32, 1, -1 do
    button = _G['GossipTitleButton' .. i]

    if button:IsVisible() then
      text = module:stripText(button:GetText())

      if button.type == 'Available' and module:shouldAccept() then
        button:Click()
      end

      if
        button.type == 'Active' and
        module:shouldComplete() and
        module.completedQuests[text]
      then
        button:Click()
      end
    end
  end
end

function module:QUEST_GREETING(...)
  if not module:shouldAutomate() then return end

  local button
  local text

  for i = 32, 1, -1 do
    button = _G['QuestTitleButton' .. i]

    if button and button:IsVisible() then
      text = module:stripText(button:GetText())
      if
        module.completedQuests[text] and
        module:shouldComplete()
      then
        button:Click()
      elseif
        not module.incompleteQuests[text] and
        module:shouldAccept()
      then
        button:Click()
      end
    end
  end
end

function module:QUEST_DETAIL()
  if not module:shouldAutomate() then return end
  if not module:shouldAccept() then return end

  AcceptQuest()
end
